#!/usr/bin/env python3
"""
Windows Performance Monitor (System Management Tool)

Parameters:
    --cpu: Show CPU usage statistics
    --memory: Show memory usage
    --disk: Show disk activity
    --network: Show network usage
    --process NAME: Monitor specific process
    --interval SECONDS: Update interval
    --format FORMAT: Output format (table/json)

Requires: psutil, win32api, tabulate packages
Warning: High sampling rates may impact system performance
"""

import os
import sys
import psutil
import time
import json
import argparse
import logging
from typing import List, Dict, Any, Optional, Tuple
from tabulate import tabulate
from datetime import datetime
import collections
import threading
import queue
import wmi

class PerformanceMonitor:
    def __init__(self, history_size: int = 60):
        self.logger = logging.getLogger(__name__)
        self.history_size = history_size
        self.cpu_history = collections.deque(maxlen=history_size)
        self.memory_history = collections.deque(maxlen=history_size)
        self.disk_history = collections.deque(maxlen=history_size)
        self.network_history = collections.deque(maxlen=history_size)
        self.alert_queue = queue.Queue()
        self.alert_thresholds = {
            'cpu_percent': 80.0,
            'memory_percent': 80.0,
            'disk_percent': 90.0
        }
        self.wmi = wmi.WMI()

    def get_cpu_info(self) -> Dict[str, Any]:
        """Get detailed CPU information."""
        cpu_info = {
            'physical_cores': psutil.cpu_count(logical=False),
            'logical_cores': psutil.cpu_count(logical=True),
            'frequency': psutil.cpu_freq(),
            'percent_per_core': psutil.cpu_percent(interval=0.1, percpu=True),
            'total_percent': psutil.cpu_percent(interval=0.1),
            'stats': psutil.cpu_stats(),
            'temperature': self._get_cpu_temperature()
        }
        self.cpu_history.append({
            'timestamp': datetime.now(),
            'percent': cpu_info['total_percent']
        })
        return cpu_info

    def get_memory_info(self) -> Dict[str, Any]:
        """Get detailed memory information."""
        virtual = psutil.virtual_memory()
        swap = psutil.swap_memory()
        memory_info = {
            'virtual': {
                'total': virtual.total,
                'available': virtual.available,
                'used': virtual.used,
                'percent': virtual.percent
            },
            'swap': {
                'total': swap.total,
                'used': swap.used,
                'free': swap.free,
                'percent': swap.percent
            }
        }
        self.memory_history.append({
            'timestamp': datetime.now(),
            'percent': virtual.percent
        })
        return memory_info

    def get_disk_info(self) -> Dict[str, Any]:
        """Get detailed disk information."""
        disk_info = {
            'partitions': [],
            'io_counters': psutil.disk_io_counters(perdisk=True)
        }
        
        for partition in psutil.disk_partitions():
            try:
                usage = psutil.disk_usage(partition.mountpoint)
                disk_info['partitions'].append({
                    'device': partition.device,
                    'mountpoint': partition.mountpoint,
                    'fstype': partition.fstype,
                    'total': usage.total,
                    'used': usage.used,
                    'free': usage.free,
                    'percent': usage.percent
                })
            except Exception as e:
                self.logger.debug(
                    f"Error getting disk usage for {partition.mountpoint}: {str(e)}"
                )
                
        # Track disk usage history
        if disk_info['partitions']:
            avg_usage = sum(p['percent'] for p in disk_info['partitions']) / len(
                disk_info['partitions']
            )
            self.disk_history.append({
                'timestamp': datetime.now(),
                'percent': avg_usage
            })
            
        return disk_info

    def get_network_info(self) -> Dict[str, Any]:
        """Get detailed network information."""
        network_info = {
            'interfaces': psutil.net_if_stats(),
            'io_counters': psutil.net_io_counters(pernic=True),
            'connections': self._get_network_connections()
        }
        
        # Calculate total network throughput
        total_bytes_sent = sum(
            nic.bytes_sent for nic in network_info['io_counters'].values()
        )
        total_bytes_recv = sum(
            nic.bytes_recv for nic in network_info['io_counters'].values()
        )
        
        self.network_history.append({
            'timestamp': datetime.now(),
            'bytes_sent': total_bytes_sent,
            'bytes_recv': total_bytes_recv
        })
        
        return network_info

    def get_process_performance(self, top_n: int = 10) -> List[Dict[str, Any]]:
        """Get performance metrics for top processes."""
        processes = []
        for proc in psutil.process_iter(['pid', 'name', 'cpu_percent', 
                                       'memory_percent']):
            try:
                proc_info = proc.info
                proc_info['cpu_percent'] = proc.cpu_percent(interval=0.1)
                proc_info['memory_mb'] = proc.memory_info().rss / (1024 * 1024)
                processes.append(proc_info)
            except (psutil.NoSuchProcess, psutil.AccessDenied, 
                    psutil.ZombieProcess):
                continue
                
        # Sort by CPU usage and get top N
        return sorted(
            processes,
            key=lambda x: x['cpu_percent'],
            reverse=True
        )[:top_n]

    def _get_cpu_temperature(self) -> Optional[float]:
        """Get CPU temperature if available."""
        try:
            temperatures = psutil.sensors_temperatures()
            if temperatures:
                # Try to find CPU temperature from various sources
                for name, entries in temperatures.items():
                    if any(name.lower().startswith(x) for x in ['cpu', 'core']):
                        return sum(temp.current for temp in entries) / len(entries)
            return None
        except Exception:
            return None

    def _get_network_connections(self) -> List[Dict[str, Any]]:
        """Get detailed network connection information."""
        connections = []
        for conn in psutil.net_connections():
            try:
                connections.append({
                    'fd': conn.fd,
                    'family': conn.family,
                    'type': conn.type,
                    'local_addr': conn.laddr,
                    'remote_addr': conn.raddr,
                    'status': conn.status,
                    'pid': conn.pid
                })
            except (psutil.NoSuchProcess, psutil.AccessDenied):
                continue
        return connections

    def check_alerts(self) -> List[Dict[str, Any]]:
        """Check for performance alerts based on thresholds."""
        alerts = []
        
        # Check CPU usage
        cpu_percent = psutil.cpu_percent(interval=0.1)
        if cpu_percent > self.alert_thresholds['cpu_percent']:
            alerts.append({
                'type': 'CPU',
                'message': f'High CPU usage: {cpu_percent}%',
                'value': cpu_percent,
                'threshold': self.alert_thresholds['cpu_percent']
            })
            
        # Check memory usage
        memory = psutil.virtual_memory()
        if memory.percent > self.alert_thresholds['memory_percent']:
            alerts.append({
                'type': 'Memory',
                'message': f'High memory usage: {memory.percent}%',
                'value': memory.percent,
                'threshold': self.alert_thresholds['memory_percent']
            })
            
        # Check disk usage
        for partition in psutil.disk_partitions():
            try:
                usage = psutil.disk_usage(partition.mountpoint)
                if usage.percent > self.alert_thresholds['disk_percent']:
                    alerts.append({
                        'type': 'Disk',
                        'message': (f'High disk usage on {partition.mountpoint}: '
                                  f'{usage.percent}%'),
                        'value': usage.percent,
                        'threshold': self.alert_thresholds['disk_percent']
                    })
            except Exception:
                continue
                
        return alerts

    def format_size(self, size: int) -> str:
        """Format size in bytes to human-readable format."""
        for unit in ['B', 'KB', 'MB', 'GB', 'TB']:
            if size < 1024.0:
                return f"{size:.2f} {unit}"
            size /= 1024.0
        return f"{size:.2f} PB"

    def display_results(self, data: Dict[str, Any], 
                       output_format: str = 'table'):
        """Display performance information in specified format."""
        if output_format == 'json':
            # Convert all numeric values to strings for better readability
            formatted_data = self._format_data_for_json(data)
            print(json.dumps(formatted_data, indent=4, default=str))
        else:
            self._display_table_format(data)

    def _format_data_for_json(self, data: Dict[str, Any]) -> Dict[str, Any]:
        """Format data for JSON output."""
        formatted = {}
        
        if 'cpu' in data:
            cpu = data['cpu']
            formatted['cpu'] = {
                'usage_total': f"{cpu['total_percent']}%",
                'usage_per_core': [f"{x}%" for x in cpu['percent_per_core']],
                'physical_cores': cpu['physical_cores'],
                'logical_cores': cpu['logical_cores'],
                'frequency': f"{cpu['frequency'].current:.2f} MHz"
            }
            if cpu['temperature']:
                formatted['cpu']['temperature'] = f"{cpu['temperature']:.1f}°C"
                
        if 'memory' in data:
            memory = data['memory']['virtual']
            formatted['memory'] = {
                'total': self.format_size(memory['total']),
                'used': self.format_size(memory['used']),
                'available': self.format_size(memory['available']),
                'percent_used': f"{memory['percent']}%"
            }
            
        if 'disk' in data:
            formatted['disk'] = {
                'partitions': [{
                    'mountpoint': p['mountpoint'],
                    'total': self.format_size(p['total']),
                    'used': self.format_size(p['used']),
                    'free': self.format_size(p['free']),
                    'percent_used': f"{p['percent']}%"
                } for p in data['disk']['partitions']]
            }
            
        if 'network' in data:
            formatted['network'] = {
                'interfaces': {},
                'total_sent': self.format_size(
                    sum(nic.bytes_sent 
                        for nic in data['network']['io_counters'].values())
                ),
                'total_received': self.format_size(
                    sum(nic.bytes_recv 
                        for nic in data['network']['io_counters'].values())
                )
            }
            
        if 'top_processes' in data:
            formatted['top_processes'] = [{
                'name': p['name'],
                'pid': p['pid'],
                'cpu_percent': f"{p['cpu_percent']:.1f}%",
                'memory': f"{p['memory_mb']:.1f} MB"
            } for p in data['top_processes']]
            
        return formatted

    def _display_table_format(self, data: Dict[str, Any]):
        """Display performance data in table format."""
        # CPU Information
        if 'cpu' in data:
            print("\n=== CPU Information ===")
            cpu = data['cpu']
            print(f"Total CPU Usage: {cpu['total_percent']}%")
            print(f"Physical Cores: {cpu['physical_cores']}")
            print(f"Logical Cores: {cpu['logical_cores']}")
            print("Per-Core Usage:")
            for i, usage in enumerate(cpu['percent_per_core']):
                print(f"  Core {i}: {usage}%")
            if cpu['temperature']:
                print(f"CPU Temperature: {cpu['temperature']:.1f}°C")
                
        # Memory Information
        if 'memory' in data:
            print("\n=== Memory Information ===")
            memory = data['memory']['virtual']
            headers = ['Total', 'Used', 'Available', 'Usage']
            memory_data = [[
                self.format_size(memory['total']),
                self.format_size(memory['used']),
                self.format_size(memory['available']),
                f"{memory['percent']}%"
            ]]
            print(tabulate(memory_data, headers=headers, tablefmt='grid'))
            
        # Disk Information
        if 'disk' in data:
            print("\n=== Disk Information ===")
            headers = ['Mount Point', 'Total', 'Used', 'Free', 'Usage']
            disk_data = [
                [
                    p['mountpoint'],
                    self.format_size(p['total']),
                    self.format_size(p['used']),
                    self.format_size(p['free']),
                    f"{p['percent']}%"
                ] for p in data['disk']['partitions']
            ]
            print(tabulate(disk_data, headers=headers, tablefmt='grid'))
            
        # Network Information
        if 'network' in data:
            print("\n=== Network Information ===")
            total_sent = sum(
                nic.bytes_sent for nic in data['network']['io_counters'].values()
            )
            total_recv = sum(
                nic.bytes_recv for nic in data['network']['io_counters'].values()
            )
            print(f"Total Data Sent: {self.format_size(total_sent)}")
            print(f"Total Data Received: {self.format_size(total_recv)}")
            
        # Top Processes
        if 'top_processes' in data:
            print("\n=== Top Processes ===")
            headers = ['PID', 'Name', 'CPU %', 'Memory (MB)']
            process_data = [
                [
                    p['pid'],
                    p['name'],
                    f"{p['cpu_percent']:.1f}",
                    f"{p['memory_mb']:.1f}"
                ] for p in data['top_processes']
            ]
            print(tabulate(process_data, headers=headers, tablefmt='grid'))

def main():
    parser = argparse.ArgumentParser(
        description='Performance Monitor - Windows performance monitoring utility')
    parser.add_argument('--interval', type=float, default=1.0,
                       help='Update interval in seconds (default: 1.0)')
    parser.add_argument('--duration', type=float,
                       help='Monitoring duration in seconds')
    parser.add_argument('--format', choices=['table', 'json'],
                       default='table',
                       help='Output format')
    parser.add_argument('--top-processes', type=int, default=10,
                       help='Number of top processes to show')
    parser.add_argument('--no-cpu', action='store_true',
                       help='Skip CPU monitoring')
    parser.add_argument('--no-memory', action='store_true',
                       help='Skip memory monitoring')
    parser.add_argument('--no-disk', action='store_true',
                       help='Skip disk monitoring')
    parser.add_argument('--no-network', action='store_true',
                       help='Skip network monitoring')
    parser.add_argument('--alerts', action='store_true',
                       help='Show performance alerts')
    
    args = parser.parse_args()
    
    # Setup logging
    logging.basicConfig(
        level=logging.INFO,
        format='%(asctime)s - %(levelname)s - %(message)s'
    )
    
    monitor = PerformanceMonitor()
    start_time = time.time()
    
    try:
        while True:
            data = {}
            
            if not args.no_cpu:
                data['cpu'] = monitor.get_cpu_info()
            if not args.no_memory:
                data['memory'] = monitor.get_memory_info()
            if not args.no_disk:
                data['disk'] = monitor.get_disk_info()
            if not args.no_network:
                data['network'] = monitor.get_network_info()
                
            data['top_processes'] = monitor.get_process_performance(
                args.top_processes
            )
            
            if args.alerts:
                alerts = monitor.check_alerts()
                if alerts:
                    print("\n=== Performance Alerts ===")
                    for alert in alerts:
                        print(f"{alert['type']}: {alert['message']}")
                        
            monitor.display_results(data, args.format)
            
            if args.duration and time.time() - start_time >= args.duration:
                break
                
            time.sleep(args.interval)
            
            # Clear screen for next update if using table format
            if args.format == 'table':
                os.system('cls' if os.name == 'nt' else 'clear')
                
    except KeyboardInterrupt:
        print("\nMonitoring stopped by user")
    except Exception as e:
        logging.error(f"An error occurred: {str(e)}")
        return 1
        
    return 0

if __name__ == "__main__":
    exit(main())
